@@ -8,9 +8,8 @@ fabfile, and executing the commands given.
 The other callables defined in this module are internal only. Anything useful
 to individuals leveraging Fabric as a library, should be kept elsewhere.
 """
-
-from collections import defaultdict
-from operator import add, isMappingType
+import getpass
+from operator import isMappingType
 from optparse import OptionParser
 import os
 import sys
@@ -20,12 +19,12 @@ import types
 from fabric import api, state, colors
 from fabric.contrib import console, files, project
 
-from fabric.network import denormalize, disconnect_all, ssh
+from fabric.network import disconnect_all, ssh
 from fabric.state import env_options
-from fabric.tasks import Task, execute
+from fabric.tasks import Task, execute, get_task_details
 from fabric.task_utils import _Dict, crawl
-from fabric.utils import abort, indent, warn
-from fabric.operations import _pty_size
+from fabric.utils import abort, indent, warn, _pty_size
+
 
 # One-time calculation of "all internal callables" to avoid doing this on every
 # check of a given fabfile callable (in is_classic_task()).
@@ -35,6 +34,7 @@ _internals = reduce(lambda x, y: x + filter(callable, vars(y).values()),
     []
 )
 
+
 # Module recursion cache
 class _ModuleCache(object):
     """
@@ -59,7 +59,7 @@ def load_settings(path):
     """
     Take given file path and return dictionary of any key=value pairs found.
 
-    Usage docs are in docs/usage/fab.rst, in "Settings files."
+    Usage docs are in sites/docs/usage/fab.rst, in "Settings files."
     """
     if os.path.exists(path):
         comments = lambda s: s and not s.startswith("#")
@@ -80,14 +80,15 @@ def _is_package(path):
     )
 
 
-def find_fabfile():
+def find_fabfile(names=None):
     """
     Attempt to locate a fabfile, either explicitly or by searching parent dirs.
 
-    Usage docs are in docs/usage/fabfiles.rst, in "Fabfile discovery."
+    Usage docs are in sites/docs/usage/fabfiles.rst, in "Fabfile discovery."
     """
-    # Obtain env value
-    names = [state.env.fabfile]
+    # Obtain env value if not given specifically
+    if names is None:
+        names = [state.env.fabfile]
     # Create .py version if necessary
     if not names[0].endswith('.py'):
         names += [names[0] + '.py']
@@ -209,8 +210,12 @@ def extract_tasks(imported_vars):
         name, obj = tup
         if is_task_object(obj):
             state.env.new_style_tasks = True
-            # Honor instance.name
-            new_style_tasks[obj.name] = obj
+            # Use instance.name if defined
+            if obj.name and obj.name != 'undefined':
+                new_style_tasks[obj.name] = obj
+            else:
+                obj.name = name
+                new_style_tasks[name] = obj
             # Handle aliasing
             if obj.aliases is not None:
                 for alias in obj.aliases:
@@ -264,7 +269,9 @@ def parse_options():
     # Initialize
     #
 
-    parser = OptionParser(usage="fab [options] <command>[:arg1,arg2=val2,host=foo,hosts='h1;h2',...] ...")
+    parser = OptionParser(
+        usage=("fab [options] <command>"
+               "[:arg1,arg2=val2,host=foo,hosts='h1;h2',...] ..."))
 
     #
     # Define options that don't become `env` vars (typically ones which cause
@@ -287,6 +294,12 @@ def parse_options():
         help="formats --list, choices: %s" % ", ".join(LIST_FORMAT_OPTIONS)
     )
 
+    parser.add_option('-I', '--initial-password-prompt',
+        action='store_true',
+        default=False,
+        help="Force password prompt up-front"
+    )
+
     # List Fab commands found in loaded fabfiles/source files
     parser.add_option('-l', '--list',
         action='store_true',
@@ -335,12 +348,14 @@ def parse_options():
     opts, args = parser.parse_args()
     return parser, opts, args
 
+
 def _is_task(name, value):
     """
     Is the object a task as opposed to e.g. a dict or int?
     """
     return is_classic_task((name, value)) or is_task_object(value)
 
+
 def _sift_tasks(mapping):
     tasks, collections = [], []
     for name, value in mapping.iteritems():
@@ -352,6 +367,7 @@ def _sift_tasks(mapping):
     collections = sorted(collections)
     return tasks, collections
 
+
 def _task_names(mapping):
     """
     Flatten & sort task names in a breadth-first fashion.
@@ -419,6 +435,7 @@ def _nested_list(mapping, level=1):
 COMMANDS_HEADER = "Available commands"
 NESTED_REMINDER = " (remember to call as module.[...].task)"
 
+
 def list_commands(docstring, format_):
     """
     Print all found commands/tasks, then exit. Invoked with ``-l/--list.``
@@ -456,10 +473,14 @@ def display_command(name):
         msg = "Task '%s' does not appear to exist. Valid task names:\n%s"
         abort(msg % (name, "\n".join(_normal_list(False))))
     # Print out nicely presented docstring if found
-    if command.__doc__:
+    if hasattr(command, '__details__'):
+        task_details = command.__details__()
+    else:
+        task_details = get_task_details(command)
+    if task_details:
         print("Displaying detailed information for task '%s':" % name)
         print('')
-        print(indent(command.__doc__, strip=True))
+        print(indent(task_details, strip=True))
         print('')
     # Or print notice if not
     else:
@@ -498,7 +519,7 @@ def parse_arguments(arguments):
     """
     Parse string list into list of tuples: command, args, kwargs, hosts, roles.
 
-    See docs/usage/fab.rst, section on "per-task arguments" for details.
+    See sites/docs/usage/fab.rst, section on "per-task arguments" for details.
     """
     cmds = []
     for cmd in arguments:
@@ -559,15 +580,12 @@ def update_output_levels(show, hide):
             state.output[key] = False
 
 
-from fabric.tasks import _parallel_tasks
-
-
 def show_commands(docstring, format, code=0):
     print("\n".join(list_commands(docstring, format)))
     sys.exit(code)
 
 
-def main():
+def main(fabfile_locations=None):
     """
     Main command-line execution loop.
     """
@@ -607,20 +625,23 @@ def main():
             if key in state.env and isinstance(state.env[key], basestring):
                 state.env[key] = state.env[key].split(',')
 
+        # Feed the env.tasks : tasks that are asked to be executed.
+        state.env['tasks'] = arguments
+
         # Handle output control level show/hide
         update_output_levels(show=options.show, hide=options.hide)
 
         # Handle version number option
         if options.show_version:
             print("Fabric %s" % state.env.version)
-            print("ssh (library) %s" % ssh.__version__)
+            print("Paramiko %s" % ssh.__version__)
             sys.exit(0)
 
         # Load settings from user settings file, into shared env dict.
         state.env.update(load_settings(state.env.rcfile))
 
         # Find local fabfile path or abort
-        fabfile = find_fabfile()
+        fabfile = find_fabfile(fabfile_locations)
         if not fabfile and not remainder_arguments:
             abort("""Couldn't find any fabfiles!
 
@@ -703,6 +724,11 @@ Remember that -f can be used to specify fabfile path, and use -h for help.""")
         if not commands_to_run and default:
             commands_to_run.append((default.name, [], {}, [], [], []))
 
+        # Initial password prompt, if requested
+        if options.initial_password_prompt:
+            prompt = "Initial value for env.password: "
+            state.env.password = getpass.getpass(prompt)
+
         if state.output.debug:
             names = ", ".join(x[0] for x in commands_to_run)
             print("Commands to run: %s" % names)
@@ -724,7 +750,7 @@ Remember that -f can be used to specify fabfile path, and use -h for help.""")
         raise
     except KeyboardInterrupt:
         if state.output.status:
-            print >> sys.stderr, "\nStopped."
+            sys.stderr.write("\nStopped.\n")
         sys.exit(1)
     except:
         sys.excepthook(*sys.exc_info())
