@@ -1,7 +1,9 @@
 from __future__ import with_statement
 
 from functools import wraps
+import inspect
 import sys
+import textwrap
 
 from fabric import state
 from fabric.utils import abort, warn, error
@@ -11,6 +13,49 @@ from fabric.job_queue import JobQueue
 from fabric.task_utils import crawl, merge, parse_kwargs
 from fabric.exceptions import NetworkError
 
+if sys.version_info[:2] == (2, 5):
+    # Python 2.5 inspect.getargspec returns a tuple
+    # instead of ArgSpec namedtuple.
+    class ArgSpec(object):
+        def __init__(self, args, varargs, keywords, defaults):
+            self.args = args
+            self.varargs = varargs
+            self.keywords = keywords
+            self.defaults = defaults
+            self._tuple = (args, varargs, keywords, defaults)
+
+        def __getitem__(self, idx):
+            return self._tuple[idx]
+
+    def patched_get_argspec(func):
+        return ArgSpec(*inspect._getargspec(func))
+
+    inspect._getargspec = inspect.getargspec
+    inspect.getargspec = patched_get_argspec
+
+
+def get_task_details(task):
+    details = [
+        textwrap.dedent(task.__doc__)
+        if task.__doc__
+        else 'No docstring provided']
+    argspec = inspect.getargspec(task)
+
+    default_args = [] if not argspec.defaults else argspec.defaults
+    num_default_args = len(default_args)
+    args_without_defaults = argspec.args[:len(argspec.args) - num_default_args]
+    args_with_defaults = argspec.args[-1 * num_default_args:]
+
+    details.append('Arguments: %s' % (
+        ', '.join(
+            args_without_defaults + [
+                '%s=%r' % (arg, default)
+                for arg, default in zip(args_with_defaults, default_args)
+            ])
+    ))
+
+    return '\n'.join(details)
+
 
 def _get_list(env):
     def inner(key):
@@ -47,6 +92,9 @@ class Task(object):
             self.name = name
         self.is_default = default
 
+    def __details__(self):
+        return get_task_details(self.run)
+
     def run(self):
         raise NotImplementedError
 
@@ -125,6 +173,9 @@ class WrappedCallableTask(Task):
     def __getattr__(self, k):
         return getattr(self.wrapped, k)
 
+    def __details__(self):
+        return get_task_details(self.wrapped)
+
 
 def requires_parallel(task):
     """
@@ -149,6 +200,10 @@ def _parallel_tasks(commands_to_run):
     ))
 
 
+def _is_network_error_ignored():
+    return not state.env.use_exceptions_for['network'] and state.env.skip_bad_hosts
+
+
 def _execute(task, host, my_env, args, kwargs, jobs, queue, multiprocessing):
     """
     Primary single-host work body of execute()
@@ -177,8 +232,7 @@ def _execute(task, host, my_env, args, kwargs, jobs, queue, multiprocessing):
             def submit(result):
                 queue.put({'name': name, 'result': result})
             try:
-                key = normalize_to_string(state.env.host_string)
-                state.connections.pop(key, "")
+                state.connections.clear()
                 submit(task.run(*args, **kwargs))
             except BaseException, e: # We really do want to capture everything
                 # SystemExit implies use of abort(), which prints its own
@@ -187,12 +241,16 @@ def _execute(task, host, my_env, args, kwargs, jobs, queue, multiprocessing):
                 # clear what host encountered the exception that will
                 # print.
                 if e.__class__ is not SystemExit:
-                    sys.stderr.write("!!! Parallel execution exception under host %r:\n" % name)
+                    if not (isinstance(e, NetworkError) and
+                            _is_network_error_ignored()):
+                        sys.stderr.write("!!! Parallel execution exception under host %r:\n" % name)
                     submit(e)
                 # Here, anything -- unexpected exceptions, or abort()
                 # driven SystemExits -- will bubble up and terminate the
                 # child process.
-                raise
+                if not (isinstance(e, NetworkError) and
+                        _is_network_error_ignored()):
+                    raise
 
         # Stuff into Process wrapper
         kwarg_dict = {
@@ -342,7 +400,10 @@ def execute(task, *args, **kwargs):
             ran_jobs = jobs.run()
             for name, d in ran_jobs.iteritems():
                 if d['exit_code'] != 0:
-                    if isinstance(d['results'], BaseException):
+                    if isinstance(d['results'], NetworkError) and \
+                            _is_network_error_ignored():
+                        error(d['results'].message, func=warn, exception=d['results'].wrapped)
+                    elif isinstance(d['results'], BaseException):
                         error(err, exception=d['results'])
                     else:
                         error(err)
